-- NoteCase Pro Script
-- Filename: DocUtils.lua
-- Version 1.0
-- Description: Stand-alone menued script with options to:
-- 		* Close all open documents, saving any docs with unsaved changes
--		* Save document copy as another document
-- 		* Put current document's path in clipboard
--    * Put paths for all open documents in clipboard, one per line
--

-- Requires NoteCase Pro >= v. 3.1.5
--
-- Credit: Close all open documents code derived in part 
-- from a script by Miroslav Rajcic, 04_CloseAll.lua
-- 
-- Rights:
-- This script's author, Paul E. Merrell, hereby waives all copyright 
-- and related or neighboring rights to this script, pursuant to the
-- Creative Commons CC0 Universal relinquishment of rights found at
-- http://creativecommons.org/publicdomain/zero/1.0/


-- BEGIN SCRIPT

-- make sure app version supports necessary scriptable program commands
nMaj, nMin, nRev = Nc_Config_GetAppVersion()
if (nMaj < 3 or (nMaj == 3 and nMin < 1) or (nMaj == 3 and nMin == 1 and nRev < 5)) then
	Nc_GUI_MessageBox("ERROR: Script requires NcPro version >= 3.1.5")
	return
end

-- =========================================================
--    INITIAL MENU
-- =========================================================

-- get user's choice of actions
nSelection = Nc_GUI_SelectionDlg("Document Utilities", "Save Document Copy As", "Copy Current Document's Path", "Copy All Open Documents' Paths", "Close All Open Documents", "Close All But Current Document", "Help")

-- check if user canceled the selection and if so abort the script
if nSelection == -1 then
	return
end


-- =========================================================
--   DEFINE FUNCTIONS
-- =========================================================

-- =========================================================
--	   BEGIN FUNCTION GETFILEPATHS
-- =========================================================

function GetFilePaths(bGetAll)
-- function to get file path for current document (bGetAll == 0)
-- or file paths for all open documents (bGetAll == 1).
-- Returns either a single file path, or all 
-- file paths with each on a separate line

	-- get current document's file path

	if bGetAll == 0 then
	
		-- get and store current document's ID as variable nDocID
		nDocID = Nc_Doc_ID_GetCur()
	
		-- get current document's filepath and store as strDocPath
		strDocPath = Nc_Doc_GetPath(nDocID)
	
		-- check to see if file path includes one or more spaces
		-- and if it does, enclose it in double quotation marks
		if not (nil == string.find(strDocPath, " ") ) then
			strDocPath = "\"" .. strDocPath .. "\""
		end
		
		-- store the file path in strFilePaths
		strFilePaths = strDocPath
	
		return strFilePaths
	end

	-- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
	-- get file paths for all open documents
	
	if bGetAll == 1 then
	
		-- create empty variable strFilePaths
		strFilePaths = ""
	
		-- get count of open documents
		nDocCount =  Nc_App_GetDocCount()

		-- for each open document ...
		for i=nDocCount-1,0,-1 do
	
			-- get the document's ID
			nDocID = Nc_Doc_ID_GetByIdx(i)

			-- get the document's filepath and store as strDocPath
			strDocPath = Nc_Doc_GetPath(nDocID)
	
			-- check to see if the file path includes one or more spaces
			-- and if it does, enclose file path in double quotation marks
			if not (nil == string.find(strDocPath, " ") ) then
				strDocPath = "\"" .. strDocPath .. "\""
			end
				
			-- append the document's file path to strFilePaths on a newline
			strFilePaths = strFilePaths .. "\n" .. strDocPath
		end
		
		return strFilePaths
	end
end -- function


-- =========================================================
--   BEGIN FUNCTION CLOSEDOCS
-- =========================================================

	-- function to close open documents
	-- call with "CloseDocs(boolean bCloseAll)"
	-- if bCloseAll == 1, closes all open documents
	-- if BCloseAll == 0, closes all but the current document
		
	function CloseDocs(bCloseAll)

		if bCloseAll == 1 then
	
			-- close all open documents

			-- get count of open documents
			nDocCount = Nc_App_GetDocCount()

			-- close all open documents
			for i=nDocCount-1,0,-1 do
				nDocID = Nc_Doc_ID_GetByIdx(i)
				Nc_Doc_Close(nDocID)
			end
		 return
		end
	
	------------------------------------------------
	
	-- close all open documents except the current document
	if bCloseAll == 0 then
		
		-- get and store current document's ID as variable nCurDocID
		nCurDocID = Nc_Doc_ID_GetCur()
				
		-- get count of open documents
		nDocCount = Nc_App_GetDocCount()

		-- close all open documents except current document
		for i=nDocCount-1,0,-1 do
				
			-- get document's ID and store as variable nDocID 
			nDocID = Nc_Doc_ID_GetByIdx(i)
				
			-- test if document is current document and close if not
			if nDocID ~= nCurDocID then
				Nc_Doc_Close(nDocID)
			end
		end
	end
 return
end -- function	

-- =========================================================
--    END FUNCTION DEFINITIONS
-- =========================================================

-- =========================================================
--    BEGIN INDIVIDUAL SCRIPTED ACTIONS
-- =========================================================

--=========================================
-- USER CHOSE SAVE DOCUMENT COPY AS

if nSelection == 0 then

	-- get and store current document's ID as variable nDocID
	nDocID = Nc_Doc_ID_GetCur()


	-- display Help
	Nc_GUI_MessageBox([[

	Document copy can be saved in .html, .ncd, .ncdb,     
	.ncz, or .txt format. Save format is determined
	by the extension you add to the file name. If
	saved as .html, document is saved in simple HTML
	format with bold headings.

	]])
	
	-- show file save dialog and store user-designated file path (including file name and extension) as variable strExportPath
	strExportPath = Nc_GUI_FileDlg(0, "", "Save Document Copy As")

	-- check if user selected cancel and if so abort the script
	if strExportPath == "" then
		return
	end
		
	-- export document to the format and location the user has chosen
	Nc_Doc_Export(nDocID, strExportPath)

	return
end
 

--=========================================
-- USER CHOSE COPY CURRENT DOCUMENT'S PATH
 
if nSelection == 1 then
	
	-- Get the current document's file path
	-- using the GetFilePaths function 
	strFilePaths = GetFilePaths(0)

	-- write file path to the clipboard in plain text
	Nc_Env_Clipboard_Set(strFilePaths, 0)
	
	-- tell the user what happened
	Nc_GUI_MessageBox("\nThe current document's file path has\nbeen written to the clipboard.\n")

	return
end

--=========================================
-- USER CHOSE COPY ALL OPEN DOCUMENTS' PATHS

if nSelection == 2 then

	-- Get all open documents' file paths 
	-- using GetFilePaths function
	strFilePaths = GetFilePaths(1)
	
	-- write file paths to the clipboard
	Nc_Env_Clipboard_Set(strFilePaths, 0)
	
	-- tell the user what happened
	Nc_GUI_MessageBox("The file paths for all open documents have been written to the clipboard.")

	return strFilePaths
end

--=========================================
-- USER CHOSE CLOSE ALL OPEN DOCUMENTS

if nSelection == 3 then 

	-- call function to close all open documents
	-- (files will be saved if they have unsaved changes)
	CloseDocs(1)
 return
end

--=========================================
-- USER CHOSE CLOSE ALL BUT CURRENT DOCUMENT

if nSelection == 4 then

	-- call function to close all but current document
	-- (files will be saved if they have unsaved changes)
	CloseDocs(0)
 return
end

--=========================================
-- USER CHOSE HELP

if nSelection == 5 then
	
	-- display Help for the script
	Nc_GUI_MessageBox([[
	
	Save Document Copy As -- Opens file dialog
	to save a copy of the current document to a
	different file name or location. Does not
	change the file name or location of the 
	current document.
	
	Copy Current Document's Path -- Writes the
	current document's path to the clipboard. If      
	the path contains spaces, it is wrapped in
	double quotation marks.
	
	Copy All Open Documents' Paths -- Writes
	the paths for all open documents to the 
	clipboard. Paths containing spaces are 
	double quoted.
	
	Close All Open Documents -- Closes all
	open documents.
	
	Close All But Current Document -- Closes 
	all open documents except the current 
	document. 
	]])
	
 return
end